<?php

/**
 * @file
 * Exception classes used by APIs.
 * @author Ahmad hejazee <mngafa@gmail.com>
 */

/**
 * Exception: When some required directives are missing in sections.
 */
class NGSLAccessByFileAPI_MissingDirectives extends \Exception {
  private $section_directives;

  /**
   * @param array $section_directives List of missing directives keyed by their section names.
   */
  public function __construct(array $section_directives) {
    $this->section_directives = $section_directives;

    // Build message.
    $message = 'Required directives are missing: ';
    foreach ($section_directives as $section => $directives) {
      $message .= '(Directives "' . implode(', ', $directives) . '"';
      $message .= ' are missing in section "[' . $section . ']") ';
    }

    parent::__construct($message);
  }

  /**
   * @return array List of missing directives keyed by their section names.
   */
  public function getMissingDirectives() {
    return $this->section_directives;
  }
}

/**
 * Exception: When a required section is missing in the .ngslaccess file
 */
class NGSLAccessByFileAPI_MissingSections extends \Exception {
  private $sections;

  /**
   * @param array $sections List of missing sections
   */
  public function __construct(array $sections) {
    $this->sections = $sections;

    // Build message.
    $message = 'Required sections are missing in the .ngslaccess file: ';
    $message .= implode(', ', $sections);

    parent::__construct($message);
  }

  public function getMissingSections() {
    return $this->sections;
  }
}

/**
 * Exception: When schema version does not match with required version
 */
class NGSLAccessByFileAPI_VersionMismatch extends \Exception {
  private $needed;
  private $given;

  public function __construct($needed, $given) {
    $this->needed = $needed;
    $this->given = $given;

    // Build message.
    $message = 'Schema version (in .ngslaccess file) does not match the required version (in API).';
    $message .= 'Needed version: ' . $needed . ', but given: ' . $given;

    parent::__construct($message);
  }

  public function getVersions() {
    return array(
      'needed' => $this->needed,
      'given' => $this->given,
    );
  }
}

/**
 * Exception: When schema api name does not match with the class' api name
 */
class NGSLAccessByFileAPI_APINameMismatch extends \Exception {
  private $needed;
  private $given;

  public function __construct($needed, $given) {
    $this->needed = $needed;
    $this->given = $given;

    // Build message.
    $message = 'API name (in .ngslaccess file) does not match the API name of the class.';
    $message .= 'Needed API name: ' . $needed . ', but given: ' . $given;

    parent::__construct($message);
  }

  public function getVersions() {
    return array(
      'needed' => $this->needed,
      'given' => $this->given,
    );
  }
}

/**
 * Exception: When directive types are incorrect in the .ngslaccess file
 * for example, needed array, given string, or vice versa.
 */
class NGSLAccessByFileAPI_InvalidTypeDirectives extends \Exception {
  private $section_invalids;

  /**
   * @param array $section_invalids List of invalid typed directives keyed by their sections
   */
  public function __construct(array $section_invalids) {
    $this->section_invalids = $section_invalids;

    // Build the message.
    $message = 'Invalid directive types are provided in .ngslaccess file: ';
    foreach ($section_invalids as $section => $invalids) {
      foreach ($invalids as $invalid) {
        $message .= '(In section "[' . $section . ']": ';
        $message .= ' Directive "' . $invalid['directive'] . '" should be ';
        $message .=  $invalid['needed_type'] . ', but ' . $invalid['given_type'] . ' given.) ';
      }
    }

    parent::__construct($message);
  }

  public function getInvalidTypedDirectives() {
    return $this->section_invalids;
  }
}

/**
 * Exception: When a reading a non-existent directive from schema
 */
class NGSLAccessByFileAPI_SchemaReadValueException extends \Exception {
  private $section;
  private $directive;

  /**
   * @param string $section
   * @param int $directive
   */
  public function __construct($section, $directive) {
    $this->section = $section;
    $this->directive = $directive;

    // Build the message.
    $message = 'Reading non-existent directive "' . $directive . '" from schema at section "[' . $section . ']"';

    parent::__construct($message);
  }

  public function getInvalidTypedDirectives() {
    return array(
      'section'   => $this->section,
      'directive' => $this->directive,
    );
  }
}

/**
 * Exception: When looking for a handler class of a particular handler name, fails.
 */
class NGSLAccessByFileAPI_HanlerClassNotFound extends \Exception {
  private $handler;

  /**
   * @param string $handler The handler name that it's class could not be found.
   */
  public function __construct($handler) {
    $this->handler = $handler;

    // Build the message.
    $message = 'Handler class not found for handler name: "' . $handler . '"';

    parent::__construct($message);
  }

  public function getNotFoundHandler() {
    return $this->handler;
  }
}

/**
 * Exception: When schema is invalid.
 */
class NGSLAccessByFileAPI_InvalidSchema extends \Exception {
  // TODO: Complete this
}
