<?php
/**
 * @author Ahmad Hejazee <mngafa@gmail.com>
 *
 * Provides the NGSLAccessByFileAPI class and its requirements.
 */

/**
 * The base class for AccessByFile APIs
 *
 * AccessByFile APIs are the way to process the rules defined in ngslaccess files and
 * evaluate them and tell whether access should be granted or not.
 *
 * Various API classes may exist and provide their own way for processing ngslaccess rules.
 */
abstract class NGSLAccessByFileAPI {
  /**
   * @var array Stores the schema parsed definition (from ngslaccess file)
   */
  protected $schema;

  /**
   * Class constructor
   *
   * @param $schema array The parsed ngslaccess schema
   */
  final public function __construct($schema) {
    $this->schema = $schema;
  }

  /**
   * Return the required version number. (should be equal to the version defined in ngslaccess file
   * with the 'version' directive.)
   *
   * @return int Required version
   */
  public function getVersion() {
    return 2;
  }

  /**
   * Get API name. e.g. 'simple' or 'logical'
   *
   * @return string API name
   */
  abstract public function getAPIName();

  /**
   * Return a list of all required directives that should be present
   * in the [ngslaccess] section.
   *
   * @return array
   */
  abstract protected function requiredDirectives();

  /**
   * Return a list of all required sections that should be present in the ngslaccess file.
   *
   * @return array
   */
  abstract protected function requiredSections();

  /**
   * Validate the ngslaccess parsed schema.
   *
   * This should validate other aspects of schema. (e.g. contents of other sections)
   * Because the [ngslaccess] and its directives and also required sections are checked
   * in other methods.
   *
   * The schema is at $this->schema.
   *
   * @throws NGSLAccessByFileAPI_InvalidSchema
   */
  abstract protected function validateSchema();

  /**
   * Validate the whole schema. Including required directives,
   * required sections, and the inner parts of the schema.
   *
   * The method is final
   *
   * @throws NGSLAccessByFileAPI_MissingSections
   * @throws NGSLAccessByFileAPI_MissingDirectives
   * @throws NGSLAccessByFileAPI_VersionMismatch
   * @throws NGSLAccessByFileAPI_APINameMismatch
   * @throws NGSLAccessByFileAPI_InvalidTypeDirectives
   * @throws NGSLAccessByFileAPI_InvalidSchema
   */
  final public function validate() {
    // Check if [ngslaccess] section is present
    // This section is required in all APIs
    if (!isset($this->schema['ngslaccess']) || !is_array($this->schema['ngslaccess'])) {
      throw new NGSLAccessByFileAPI_MissingSections(array('ngslaccess'));
    }

    $missing = array();
    // Check for 'version' and 'api' directives in [ngslaccess] section.
    foreach (array('version', 'api') as $directive) {
      // These directives should not be array (should be string)
      if (!isset($this->schema['ngslaccess'][$directive]) || is_array($this->schema['ngslaccess'][$directive])) {
        // Add the directive to the list of missing directives.
        $missing['ngslaccess'][] = $directive;
      }
    }
    if (!empty($missing)) {
      throw new NGSLAccessByFileAPI_MissingDirectives($missing);
    }

    // Check if version is supported
    $needed_ver = $this->getVersion();
    $given_ver = $this->schema['ngslaccess']['version'];
    if ($given_ver != $needed_ver) {
      throw new NGSLAccessByFileAPI_VersionMismatch($needed_ver, $given_ver);
    }

    // Check if api name is correct
    $needed_api = $this->getAPIName();
    $given_api = $this->schema['ngslaccess']['api'];
    if ($given_api != $needed_api) {
      throw new NGSLAccessByFileAPI_APINameMismatch($needed_api, $given_api);
    }

    // Check for other required directives in [ngslaccess] section
    // as defined in requiredDirectives() method.
    $directives = $this->requiredDirectives();
    $missing = array();
    foreach ($directives as $directive => $options) {
      if (!isset($this->schema['ngslaccess'][$directive])) {
        // Add the directive to the list of missing directives.
        $missing['ngslaccess'][] = $directive;
      }
    }

    // If there exists some missing directives, throw exception.
    if (!empty($missing)) {
      throw new NGSLAccessByFileAPI_MissingDirectives($missing);
    }

    // Check required directive types in [ngslaccess] section
    // as defined in requiredDirectives() method.
    $invalids = array();
    foreach ($directives as $directive => $options) {
      $needs_array = (isset($options['type']) && ($options['type'] == 'array'));
      $array_given = is_array($this->schema['ngslaccess'][$directive]);

      if ($needs_array xor $array_given) {
        $invalids['ngslaccess'][] = array(
          'directive'   => $directive,
          'needed_type' => isset($options['type']) ? $options['type'] : 'string',
          'given_type'  => $array_given ? 'array' : 'string',
        );
      }
    }

    // If there exists some invalid typed directives, throw exception.
    if (!empty($invalids)) {
      throw new NGSLAccessByFileAPI_InvalidTypeDirectives($invalids);
    }

    // Check for required sections
    $sections = $this->requiredSections();
    $missing = array();
    foreach ($sections as $section) {
      if (!isset($this->schema[$section]) || !is_array($this->schema[$section])) {
        // Add the directive to the list of missing directives.
        $missing[] = $section;
      }
    }

    // If there exists some missing sections, throw exception.
    if (!empty($missing)) {
      throw new NGSLAccessByFileAPI_MissingSections($missing);
    }

    // Now validate the contents of the schema.
    // This function also may throw exceptions.
    $this->validateSchema();
  }

  /**
   * Run the API and check all criteria
   *
   * @return bool Whether the access is granted or not.
   *
   * @throws NGSLAccessByFileAPI_HanlerClassNotFound
   * @throws NGSLAccessByFileAPI_SchemaReadValueException
   */
  abstract public function hasAccess();

  /**
   * Read value from schema.
   *
   * This method is intended to be called from hasAccess() method, after validation.
   *
   * @param string $directive The name of directive or value to fetch
   * @param string $section The section name
   *
   * @return mixed
   *
   * @throws NGSLAccessByFileAPI_SchemaReadValueException
   */
  final protected function get($directive, $section = 'ngslaccess') {
    if (!isset($this->schema[$section][$directive])) {
      throw new NGSLAccessByFileAPI_SchemaReadValueException($section, $directive);
    }

    return $this->schema[$section][$directive];
  }
}
