<?php

/**
 * @file
 * Exception classes used by Logic API.
 * @author Ahmad hejazee <mngafa@gmail.com>
 */

/**
 * Exception: When a rule has unsupported type.
 */
class NGSLAccessByFileAPILogic_UnsupportedTypeRule extends \Exception {
  private $needed;
  private $given;

  /**
   * @param array $needed
   * @param array $given
   */
  public function __construct(array $needed, array $given) {
    $this->needed  = $needed;
    $this->given   = $given;

    // Build message.
    $message = 'Invalid rule types in .ngslaccess: ';
    foreach ($given as $section => $type) {
      $message .= "(In section '[{$section}]': rule type is '{$type}'.) ";
    }

    $message .= ' Supported types are "' . implode(', ', $needed) . '".';

    parent::__construct($message);
  }

  public function getRuleTypes() {
    return array(
      'needed'  => $this->needed,
      'given'   => $this->given,
    );
  }
}

/**
 * Exception: When a rule has invalid operator or operator with invalid number of arguments.
 */
class NGSLAccessByFileAPILogic_InvalidOperators extends \Exception {
  private $invalid_operators;

  public function __construct(array $invalid_operators) {
    $this->invalid_operators = $invalid_operators;

    // Build message;
    $message = 'Invalid operators supplied for Logic API rules: ';

    foreach ($invalid_operators as $section_name => $section) {
      foreach ($section as $op_name => $op) {
        $message .= "(In section '[{$section_name}]', operator '{$op_name}' ";

        if ($op['op_valid']) {
          $min = $op['needed_args_min'];
          $max = $op['needed_args_max'];
          $cnt = $op['op_given_args_cnt'];

          if ($max == 0) {
            $message .= "requires at least {$min} arguments or more";
          }
          elseif ($max = $min) {
            $message .= "requires exactly {$min} arguments";
          }
          else {
            $message .= "requires at least {$min} arguments and at most {$max} arguments";
          }

          $message .= ", but {$cnt} given.";
        }
        else {
          $message .= 'is invalid and not supported.) ';
        }
      }
    }

    parent::__construct($message);
  }

  public function getInvalidOperators() {
    return $this->invalid_operators;
  }
}

/**
 * Exception: When a logical rule has an invalid target (argument).
 */
class NGSLAccessByFileAPILogic_InvalidRuleTargets extends \Exception {
  private $inavlid_targets;

  public function __construct(array $invalid_targets) {
    $this->inavlid_targets = $invalid_targets;

    // Build message
    $message = 'Invalid rule targets found in .ngslaccess file: ';

    foreach ($invalid_targets as $section_name => $item) {
      foreach ($item as $target_name => $invalid) {
        $message .= "(In section '[{$section_name}]', target '{$target_name}' is invalid: ";

        if ($invalid['type'] == 'recursive') {
          $message .= " points to itself, however recursive pointing is dissalowed.) ";
        }
        else {
          $message .= " points to non-existent rule or handler.) ";
        }
      }
    }

    parent::__construct($message);
  }

  public function getInvalidRuleTargets() {
    return $this->inavlid_targets;
  }
}

/**
 * Exception: When the main rule in [ngslaccess] section is not a defined rule
 */
class NGSLAccessByFileAPILogic_InvalidMainRule extends Exception {
  private $main_rule;

  public function __construct($main_rule) {
    $this->main_rule = $main_rule;

    // Build message
    $message = 'In .ngslaccess file, main rule in [ngslaccess] section is invalid: "' . $main_rule . '".';

    parent::__construct($message);
  }

  public function getMainRule() {
    return $this->main_rule;
  }
}
