<?php
/**
 * @author Ahmad Hejazee <mngafa@gmail.com>
 */

/**
 * Logic api provides logical operators for processing and calculating access
 */
class NGSLAccessByFileAPILogic extends NGSLAccessByFileAPI {

  /**
   * Return API name
   */
  public function getAPIName() {
    return 'logic';
  }

  /**
   * Requires string 'rule' directive be present in [ngslaccess] section
   */
  protected function requiredDirectives() {
    return array(
      'rule'   => array('type' => 'string'),
    );
  }

  /**
   * Does not require any sections
   */
  protected function requiredSections() {
    return array();
  }

  /**
   * Validate schema.
   *
   * @throws \NGSLAccessByFileAPILogic_InvalidMainRule
   * @throws \NGSLAccessByFileAPILogic_InvalidOperators
   * @throws \NGSLAccessByFileAPILogic_InvalidRuleTargets
   * @throws \NGSLAccessByFileAPILogic_UnsupportedTypeRule
   * @throws \NGSLAccessByFileAPI_InvalidTypeDirectives
   * @throws \NGSLAccessByFileAPI_MissingDirectives
   */
  protected function validateSchema() {
    // Validate rule types for all rule sections
    $this->_validate_rule_types();

    // Validate op and args directives in all rule sections
    $this->_validate_rule_op_args();

    // Validate handler and arguments directives in all handler sections
    $this->_validate_handler_handler_and_arguments();

    // Validate rule sections have valid operator with true number of arguments
    $this->_validate_rule_operators();

    // Validate rule sections have valid argument names
    // Arguments should be defined as another rule or a handler.
    $this->_validate_rule_arguments();

    // Validate main rule as defined in [ngslaccess] section be a valid rule
    $main_rule = $this->getRules($this->schema['ngslaccess']['rule']);
    if (!$main_rule) {
      throw new NGSLAccessByFileAPILogic_InvalidMainRule($this->schema['ngslaccess']['rule']);
    }
  }

  /**
   * @return bool
   * @throws \NGSLAccessByFileAPI_HanlerClassNotFound
   * @throws \NGSLAccessByFileAPI_SchemaReadValueException
   * @throws \NGSLAccessByFileHandler_MissingArgument
   */
  public function hasAccess() {
    $main_rule = $this->schema['ngslaccess']['rule'];
    $result = $this->runRuleRecursive($main_rule);

    return $result;
  }

  /**
   * Run rule recursively
   *
   * @param $rule_name
   * @return bool
   * @throws \NGSLAccessByFileAPI_HanlerClassNotFound
   */
  private function runRuleRecursive($rule_name) {
    $rule = $this->getRules($rule_name);

    // Process rule args
    $rule->args_processed = array();
    foreach ($rule->args as $arg) {
      // Check if arg is a rule or handler
      if ($target = $this->getRules($arg)) {
        // argument is a rule. process recursively.
        $rule->args_processed[] = $this->runRuleRecursive($target->name);
      }
      elseif ($target = $this->getHandlers($arg)) {
        // argument is a handler
        $handler_class = ngsl_access_byfile_handlers_get($target->handler);

        if (!$handler_class) {
          throw new NGSLAccessByFileAPI_HanlerClassNotFound($target->handler);
        }

        $handler = new $handler_class();
        $rule->args_processed[] = $this->runHandler($handler, $target->arguments);
      }
    }

    // Apply operator on processed args.
    $result = FALSE;
    switch ($rule->op) {
      case 'not':
        $result = (!$rule->args_processed[0]);

        break;
      case 'and':
        $result = TRUE;
        foreach ($rule->args_processed as $item) {
          $result = ($result && $item);
        }

        break;
      case 'or':
        foreach ($rule->args_processed as $item) {
          $result = ($result || $item);
        }

        break;
      case 'xor':
        $result = (($rule->args_processed[0]) xor ($rule->args_processed[1]));

        break;
      case 'xnor':
        $result = !(($rule->args_processed[0]) xor ($rule->args_processed[1]));

        break;
    }

    return $result;
  }

  /**
   * Run the access handler and return the boolean result.
   *
   * @param \NGSLAccessByFileHandler $handler
   * @param array $args
   *
   * @return bool Whether the access is granted or not
   * @throws \NGSLAccessByFileHandler_MissingArgument
   */
  private function runHandler(NGSLAccessByFileHandler $handler, array $args) {
    $handler->addArguments($args);
    $result = $handler->run();

    return $result;
  }

  /**
   * Get all rules defined in schema in a considerable format.
   *
   * @param string $rule_name If provided, return only the specified rule (if exists) or FALSE.
   * @return array|\stdClass|FALSE
   * If all rules are requested, return array of all rules.
   * If one rule is requested, return stdClass of selected rule.
   * If rule does not exist, return FALSE.
   */
  private function getRules($rule_name = NULL) {
    $result = FALSE;

    if (is_null($rule_name)) {
      // get all rules.
      foreach ($this->schema as $section_name => $section) {
        if (is_array($section) && (substr($section_name, 0, 5) == 'rule-')) {
          $rule = substr($section_name, 5);
          $result[$rule] = (object)$this->schema[$section_name];
          $result[$rule]->name = $rule;
          $result[$rule]->section_name = $section_name;
        }
      }
    }
    else {
      $rules = $this->getRules();
      $result = isset($rules[$rule_name]) ? $rules[$rule_name] : FALSE;
    }

    return $result;
  }

  /**
   * Get all handlers defined in schema in a considerable format.
   *
   * @param string $handler_name If provided, return only the specified handler (if exists) or FALSE.
   * @return array|\stdClass|FALSE
   * If all handlers are requested, return array of all handlers.
   * If one handler is requested, return stdClass of selected handler.
   * If handler does not exist, return FALSE.
   */
  private function getHandlers($handler_name = NULL) {
    $result = FALSE;

    if (is_null($handler_name)) {
      // get all handlers.
      foreach ($this->schema as $section_name => $section) {
        if (is_array($section) && (substr($section_name, 0, 8) == 'handler-')) {
          $handler = substr($section_name, 8);
          $result[$handler] = (object)$this->schema[$section_name];
          $result[$handler]->name = $handler;
          $result[$handler]->section_name = $section_name;
        }
      }
    }
    else {
      $handlers = $this->getHandlers();
      $result = isset($handlers[$handler_name]) ? $handlers[$handler_name] : FALSE;
    }

    return $result;
  }

  /**
   * Get a list of rule type that can be specified in rule sections.
   *
   * @return array
   */
  public function getSupportedRuleTypes() {
    return array('logic');
  }

  /**
   * Validate all rule sections must have a supported type.
   *
   * @throws \NGSLAccessByFileAPILogic_UnsupportedTypeRule
   */
  private function _validate_rule_types() {
    $rules = $this->getRules();
    $rule_types = $this->getSupportedRuleTypes();

    // Validate rule types.
    $invalid_rule_types = array();
    foreach ($rules as $rule) {
      // Rule type should be supported.
      if (!in_array($rule->type, $rule_types)) {
        $invalid_rule_types[$rule->section_name] = $rule->type;
      }
    }
    if (!empty($invalid_rule_types)) {
      throw new NGSLAccessByFileAPILogic_UnsupportedTypeRule($rule_types, $invalid_rule_types);
    }
  }

  /**
   * Validate all rule sections must have correct op and args directives.
   *
   * @throws \NGSLAccessByFileAPI_InvalidTypeDirectives
   * @throws \NGSLAccessByFileAPI_MissingDirectives
   */
  private function _validate_rule_op_args() {
    $rules = $this->getRules();

    // validate op and args directives presence.
    $missing_directives = array();
    foreach ($rules as $rule) {
      foreach (array('op', 'args') as $directive) {
        if (!isset($rule->$directive)) {
          $missing_directives[$rule->section_name][] = $directive;
        }
      }
    }
    if (!empty($missing_directives)) {
      throw new NGSLAccessByFileAPI_MissingDirectives($missing_directives);
    }

    // Validate op and args directives types
    $invalid_typed_directives = array();
    foreach ($rules as $rule) {
      // op
      if (is_array($rule->op)) {
        $invalid_typed_directives[$rule->section_name][] = array(
          'directive' => 'op',
          'needed_type' => 'string',
          'given_type' => 'array',
        );
      }

      // args
      if (!is_array($rule->args)) {
        $invalid_typed_directives[$rule->section_name][] = array(
          'directive' => 'args',
          'needed_type' => 'array',
          'given_type' => 'string',
        );
      }
    }
    if (!empty($invalid_typed_directives)) {
      throw new NGSLAccessByFileAPI_InvalidTypeDirectives($invalid_typed_directives);
    }
  }

  /**
   * Basic validation: handler and arguments directives presence and types.
   *
   * @throws \NGSLAccessByFileAPI_InvalidTypeDirectives
   * @throws \NGSLAccessByFileAPI_MissingDirectives
   */
  private function _validate_handler_handler_and_arguments() {
    $handlers = $this->getHandlers();

    // validate handler and arguments directives presence.
    $missing_directives = array();
    foreach ($handlers as $handler) {
      foreach (array('handler', 'arguments') as $directive) {
        if (!isset($handler->$directive)) {
          $missing_directives[$handler->section_name][] = $directive;
        }
      }
    }
    if (!empty($missing_directives)) {
      throw new NGSLAccessByFileAPI_MissingDirectives($missing_directives);
    }

    // Validate handler and arguments directives types
    $invalid_typed_directives = array();
    foreach ($handlers as $handler) {
      // handler
      if (is_array($handler->handler)) {
        $invalid_typed_directives[$handler->section_name][] = array(
          'directive' => 'handler',
          'needed_type' => 'string',
          'given_type' => 'array',
        );
      }

      // arguments
      if (!is_array($handler->arguments)) {
        $invalid_typed_directives[$handler->section_name][] = array(
          'directive' => 'arguments',
          'needed_type' => 'array',
          'given_type' => 'string',
        );
      }
    }
    if (!empty($invalid_typed_directives)) {
      throw new NGSLAccessByFileAPI_InvalidTypeDirectives($invalid_typed_directives);
    }
  }

  /**
   * Validate rule sections must have valid operator with true number of arguments
   *
   * @throws \NGSLAccessByFileAPILogic_InvalidOperators
   */
  private function _validate_rule_operators() {
    $rules = $this->getRules();

    $invalid_operators = array();
    foreach ($rules as $rule) {
      $operator = $this->getOperators($rule->op);
      if ($operator) {
        // check number of arguments.
        $args_count = count($rule->args);
        if (($args_count < $operator->arg_count_min) || (($operator->arg_count_max > 0) && ($args_count > $operator->arg_count_max))) {
          // Invalid number of arguments
          $invalid_operators[$rule->section_name][$rule->op] = array(
            'op_valid' => TRUE,
            'op_given_args_cnt' => $args_count,
            'needed_args_max' => $operator->arg_count_max,
            'needed_args_min' => $operator->arg_count_min,
          );
        }
      }
      else {
        // Operator is invalid (not defined)
        $invalid_operators[$rule->section_name][$rule->op] = array(
          'op_valid' => FALSE,
        );
      }
    }

    if (!empty($invalid_operators)) {
      throw new NGSLAccessByFileAPILogic_InvalidOperators($invalid_operators);
    }
  }

  /**
   * Validate rule sections have valid argument names
   * Arguments should be defined as another rule or a handler.
   *
   * @throws \NGSLAccessByFileAPILogic_InvalidRuleTargets
   */
  private function _validate_rule_arguments() {
    $rules = $this->getRules();

    $invalid_targets = array();

    foreach ($rules as $rule) {
      foreach ($rule->args as $arg) {
        // $arg should be either defined as a rule (except the current rule) or a handler
        if ($arg == $rule->name) {
          // recursive pointing is not allowed.
          $invalid_targets[$rule->section_name][$arg] = array('type' => 'recursive');

          continue;
        }

        if ((!$this->getRules($arg)) && (!$this->getHandlers($arg))) {
          // Invalid reference.
          $invalid_targets[$rule->section_name][$arg] = array('type' => 'no_target');
        }
      }
    }

    if ($invalid_targets) {
      throw new NGSLAccessByFileAPILogic_InvalidRuleTargets($invalid_targets);
    }
  }

  /**
   * Get list of operators (array) or a single operator (object)
   *
   * @param string $op
   * @return array|stdClass|FALSE
   */
  protected function getOperators($op = NULL) {
    if (is_null($op)) {
      $operators = array();

      $operators['not'] = array(
        'description' => 'Reverse a logical value.',
        'arg_count_min' => 1,
        'arg_count_max' => 1,
      );

      $operators['and'] = array(
        'description' => 'Logical AND between two or more values.',
        'arg_count_min' => 2,
        'arg_count_max' => 0,
      );

      $operators['or'] = array(
        'description' => 'Logical OR between two or more values.',
        'arg_count_min' => 2,
        'arg_count_max' => 0,
      );

      $operators['xor'] = array(
        'description' => 'Logical XOR between two values.',
        'arg_count_min' => 2,
        'arg_count_max' => 2,
      );

      $operators['xnor'] = array(
        'description' => 'Logical XNOR between two values.',
        'arg_count_min' => 2,
        'arg_count_max' => 2,
      );

      return $operators;
    }
    else {
      $operators = $this->getOperators();

      return isset($operators[$op]) ? (object)$operators[$op] : FALSE;
    }
  }
}
